// SPDX-FileCopyrightText: 2023 Andrei Zisu <matzipan@gmail.com>
// SPDX-FileCopyrightText: 2023 Helge Eichhorn <git@helgeeichhorn.de>
//
// SPDX-License-Identifier: MPL-2.0

/// High-level ephemeris API for [`super::parser::Spk`] files.
pub mod api;
/// Low-level DAF/SPK binary file parser.
pub mod parser;
